package org.jeecg.modules.adweb.seo.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.adweb.seo.entity.SeoKeywordsSerp;
import org.jeecg.modules.adweb.seo.service.ISeoKeywordsSerpService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.Arrays;

/**
 * @Description: SEO关键词搜索排名 @Author: jeecg-boot @Date: 2024-10-15 @Version: V1.0
 */
@Tag(name = "SEO关键词搜索排名")
@RestController
@RequestMapping("/serp/seoKeywordsSerp")
@Slf4j
public class SeoKeywordsSerpController
        extends JeecgController<SeoKeywordsSerp, ISeoKeywordsSerpService> {
    @Autowired private ISeoKeywordsSerpService seoKeywordsSerpService;

    /**
     * 分页列表查询
     *
     * @param seoKeywordsSerp
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    // @AutoLog(value = "SEO关键词搜索排名-分页列表查询")
    @Operation(summary = "SEO关键词搜索排名-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<SeoKeywordsSerp>> queryPageList(
            SeoKeywordsSerp seoKeywordsSerp,
            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
            HttpServletRequest req) {
        QueryWrapper<SeoKeywordsSerp> queryWrapper =
                QueryGenerator.initQueryWrapper(seoKeywordsSerp, req.getParameterMap());
        Page<SeoKeywordsSerp> page = new Page<SeoKeywordsSerp>(pageNo, pageSize);
        IPage<SeoKeywordsSerp> pageList = seoKeywordsSerpService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param seoKeywordsSerp
     * @return
     */
    @AutoLog(value = "SEO关键词搜索排名-添加")
    @Operation(summary = "SEO关键词搜索排名-添加")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords_serp:add')")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody SeoKeywordsSerp seoKeywordsSerp) {
        seoKeywordsSerpService.save(seoKeywordsSerp);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param seoKeywordsSerp
     * @return
     */
    @AutoLog(value = "SEO关键词搜索排名-编辑")
    @Operation(summary = "SEO关键词搜索排名-编辑")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords_serp:edit')")
    @RequestMapping(
            value = "/edit",
            method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody SeoKeywordsSerp seoKeywordsSerp) {
        seoKeywordsSerpService.updateById(seoKeywordsSerp);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "SEO关键词搜索排名-通过id删除")
    @Operation(summary = "SEO关键词搜索排名-通过id删除")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords_serp:delete')")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        seoKeywordsSerpService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "SEO关键词搜索排名-批量删除")
    @Operation(summary = "SEO关键词搜索排名-批量删除")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords_serp:deleteBatch')")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.seoKeywordsSerpService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    // @AutoLog(value = "SEO关键词搜索排名-通过id查询")
    @Operation(summary = "SEO关键词搜索排名-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<SeoKeywordsSerp> queryById(
            @RequestParam(name = "id", required = true) String id) {
        SeoKeywordsSerp seoKeywordsSerp = seoKeywordsSerpService.getById(id);
        if (seoKeywordsSerp == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(seoKeywordsSerp);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param seoKeywordsSerp
     */
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords_serp:exportXls')")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, SeoKeywordsSerp seoKeywordsSerp) {
        return super.exportXls(request, seoKeywordsSerp, SeoKeywordsSerp.class, "SEO关键词搜索排名");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords_serp:importExcel')")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, SeoKeywordsSerp.class);
    }
}
