package org.jeecg.modules.adweb.seo.controller;

import java.util.Arrays;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.adweb.seo.entity.SeoPlanSubscription;
import org.jeecg.modules.adweb.seo.service.ISeoPlanSubscriptionService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.springframework.security.access.prepost.PreAuthorize;

 /**
 * @Description: seo套餐订购
 * @Author: jeecg-boot
 * @Date:   2024-10-15
 * @Version: V1.0
 */
@Tag(name="seo套餐订购")
@RestController
@RequestMapping("/serp/seoPlanSubscription")
@Slf4j
public class SeoPlanSubscriptionController extends JeecgController<SeoPlanSubscription, ISeoPlanSubscriptionService> {
	@Autowired
	private ISeoPlanSubscriptionService seoPlanSubscriptionService;
	
	/**
	 * 分页列表查询
	 *
	 * @param seoPlanSubscription
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "seo套餐订购-分页列表查询")
	@Operation(summary="seo套餐订购-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<SeoPlanSubscription>> queryPageList(SeoPlanSubscription seoPlanSubscription,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<SeoPlanSubscription> queryWrapper = QueryGenerator.initQueryWrapper(seoPlanSubscription, req.getParameterMap());
		Page<SeoPlanSubscription> page = new Page<SeoPlanSubscription>(pageNo, pageSize);
		IPage<SeoPlanSubscription> pageList = seoPlanSubscriptionService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param seoPlanSubscription
	 * @return
	 */
	@AutoLog(value = "seo套餐订购-添加")
	@Operation(summary="seo套餐订购-添加")
	@PreAuthorize("@jps.requiresPermissions('serp:seo_plan_subscription:add')")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody SeoPlanSubscription seoPlanSubscription) {
		seoPlanSubscriptionService.save(seoPlanSubscription);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param seoPlanSubscription
	 * @return
	 */
	@AutoLog(value = "seo套餐订购-编辑")
	@Operation(summary="seo套餐订购-编辑")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_plan_subscription:edit')")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody SeoPlanSubscription seoPlanSubscription) {
		seoPlanSubscriptionService.updateById(seoPlanSubscription);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "seo套餐订购-通过id删除")
	@Operation(summary="seo套餐订购-通过id删除")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_plan_subscription:delete')")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		seoPlanSubscriptionService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "seo套餐订购-批量删除")
	@Operation(summary="seo套餐订购-批量删除")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_plan_subscription:deleteBatch')")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.seoPlanSubscriptionService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "seo套餐订购-通过id查询")
	@Operation(summary="seo套餐订购-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<SeoPlanSubscription> queryById(@RequestParam(name="id",required=true) String id) {
		SeoPlanSubscription seoPlanSubscription = seoPlanSubscriptionService.getById(id);
		if(seoPlanSubscription==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(seoPlanSubscription);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param seoPlanSubscription
    */
    @PreAuthorize("@jps.requiresPermissions('serp:seo_plan_subscription:exportXls')")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, SeoPlanSubscription seoPlanSubscription) {
        return super.exportXls(request, seoPlanSubscription, SeoPlanSubscription.class, "seo套餐订购");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @PreAuthorize("@jps.requiresPermissions('serp:seo_plan_subscription:importExcel')")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, SeoPlanSubscription.class);
    }

}
