package org.jeecg.modules.adweb.theme.controller;

import java.util.Arrays;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.adweb.theme.entity.AdwebTheme;
import org.jeecg.modules.adweb.theme.service.IAdwebThemeService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.springframework.security.access.prepost.PreAuthorize;
import org.jeecg.modules.adweb.theme.dto.AdwebTemplateTagDto;

 /**
 * @Description: adweb主题库
 * @Author: jeecg-boot
 * @Date:   2024-09-30
 * @Version: V1.0
 */
@Tag(name="adweb主题库")
@RestController
@RequestMapping("/adweb/adwebTheme")
@Slf4j
public class AdwebThemeController extends JeecgController<AdwebTheme, IAdwebThemeService> {
	@Autowired
	private IAdwebThemeService adwebThemeService;
	
	/**
	 * 分页列表查询
	 *
	 * @param adwebTheme
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "adweb主题库-分页列表查询")
	@Operation(summary="adweb主题库-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<AdwebTheme>> queryPageList(AdwebTheme adwebTheme,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<AdwebTheme> queryWrapper = QueryGenerator.initQueryWrapper(adwebTheme, req.getParameterMap());
		Page<AdwebTheme> page = new Page<AdwebTheme>(pageNo, pageSize);
		IPage<AdwebTheme> pageList = adwebThemeService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param adwebTheme
	 * @return
	 */
	@AutoLog(value = "adweb主题库-添加")
	@Operation(summary="adweb主题库-添加")
	@PreAuthorize("@jps.requiresPermissions('adweb:adweb_theme:add')")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody AdwebTheme adwebTheme) {
		adwebThemeService.save(adwebTheme);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param adwebTheme
	 * @return
	 */
	@AutoLog(value = "adweb主题库-编辑")
	@Operation(summary="adweb主题库-编辑")
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_theme:edit')")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody AdwebTheme adwebTheme) {
		adwebThemeService.updateById(adwebTheme);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "adweb主题库-通过id删除")
	@Operation(summary="adweb主题库-通过id删除")
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_theme:delete')")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		adwebThemeService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "adweb主题库-批量删除")
	@Operation(summary="adweb主题库-批量删除")
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_theme:deleteBatch')")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.adwebThemeService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "adweb主题库-通过id查询")
	@Operation(summary="adweb主题库-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<AdwebTheme> queryById(@RequestParam(name="id",required=true) String id) {
		AdwebTheme adwebTheme = adwebThemeService.getById(id);
		if(adwebTheme==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(adwebTheme);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param adwebTheme
    */
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_theme:exportXls')")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, AdwebTheme adwebTheme) {
        return super.exportXls(request, adwebTheme, AdwebTheme.class, "adweb主题库");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_theme:importExcel')")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, AdwebTheme.class);
    }

	 /**
	  *
	  * @return
	  */
	 @GetMapping(value = "/getTemplateTags")
	 public Result<?> getTemplateTags() {


		 return Result.OK("");
	 }

}
